class Taskell < Formula
  desc "Command-line Kanban board/task manager with support for Trello"
  homepage "https://taskell.app"
  # TODO: Try to switch `ghc@9.2` to `ghc` when a release supports brick>=1
  url "https://github.com/smallhadroncollider/taskell/archive/refs/tags/1.11.4.tar.gz"
  sha256 "0d4f3f54fb0b975f969d7ef8a810bbc7a78e0b46aec28cc4cb337ee36e8abdfc"
  license "BSD-3-Clause"

  deprecate! date: "2024-04-05", because: :repo_archived

  depends_on "cabal-install" => :build
  depends_on "ghc@9.2" => :build
  depends_on "hpack" => :build

  uses_from_macos "ncurses"
  uses_from_macos "zlib"

  def install
    # Work around build failure from Brick v1 API.
    # src/Taskell.hs:64:13: error:
    #     Not in scope: 'Brick.continue'
    #     Module 'Brick' does not export 'continue'.
    # Issue ref: https://github.com/smallhadroncollider/taskell/issues/125
    cabal_install_constraints = ["--constraint=brick<1", "--constraint=vty<6"]

    system "hpack"
    system "cabal", "v2-update"
    system "cabal", "v2-install", *std_cabal_v2_args, *cabal_install_constraints
  end

  test do
    (testpath/"test.md").write <<~EOS
      ## To Do

      - A thing
      - Another thing
    EOS

    expected = <<~EOS
      test.md
      Lists: 1
      Tasks: 2
    EOS

    assert_match expected, shell_output("#{bin}/taskell -i test.md")
  end
end
